@@ -1,4 +1,4 @@
-__version__ = "0.5.17"
+__version__ = "0.5.18"
 
 from .core import (
     magicclass,
@@ -15,7 +15,8 @@ from .wrappers import (
     click,
     set_design,
     do_not_record,
-    bind_key
+    bind_key,
+    confirm,
     )
 
 from .fields import field, vfield
