@@ -187,6 +187,9 @@ class MagicTemplate:
     def _unify_label_widths(self):
         raise NotImplementedError()
     
+    def reset_choices(self, *args):
+        raise NotImplementedError()
+    
     @property
     def macro(self) -> GuiMacro:
         if self.__magicclass_parent__ is None:
@@ -199,12 +202,15 @@ class MagicTemplate:
         """
         Return napari.Viewer if magic class is a dock widget of a viewer.
         """
+        parent_self = self._search_parent_magicclass()
+        if parent_self.parent is None:
+            return None
         try:
             from napari.utils._magicgui import find_viewer_ancestor
         except ImportError:
             return None
-        parent_self = self._search_parent_magicclass()
-        return find_viewer_ancestor(parent_self.native)
+        viewer = find_viewer_ancestor(parent_self.native)
+        return viewer
     
     @property
     def parent_dock_widget(self) -> "QDockWidget" | None:
@@ -234,7 +240,8 @@ class MagicTemplate:
     def wraps(cls, 
               method: Callable | None = None,
               *, 
-              template: Callable | None = None) -> Callable:
+              template: Callable | None = None,
+              copy: bool = False) -> Callable:
         """
         Wrap a parent method in a child magic-class. Wrapped method will appear in 
         the child widget but behaves as if it is in the parent widget.
@@ -263,6 +270,9 @@ class MagicTemplate:
         Callable
             Same method as input, but has updated signature to hide the button.
         """      
+        if (not copy) and get_additional_option(method, "into", None) is not None:
+            # If method is already wrapped, wraps should create a copy.
+            copy = True
             
         def wrapper(method: Callable):
             # Base function to get access to the original function
@@ -273,16 +283,37 @@ class MagicTemplate:
                 
             if template is not None:
                 wraps(template)(func)
-            if hasattr(cls, func.__name__):
-                getattr(cls, func.__name__).__signature__ = get_signature(func)
             
-            upgrade_signature(func, additional_options={"into": cls.__name__})
+            predefined = getattr(cls, func.__name__, None)
+            if predefined is not None:
+                # update signature to the parent one.
+                parent_sig = get_signature(func)
+                if not hasattr(predefined, "__signature__"):
+                    predefined.__signature__ = parent_sig
+                else:
+                    sig: inspect.Signature = predefined.__signature__
+                    predefined.__signature__ = sig.replace(
+                        parameters=parent_sig.parameters.values(), 
+                        return_annotation=parent_sig.return_annotation
+                        )
+                upgrade_signature(predefined, additional_options={"copyto": []})
+            
+            if copy:
+                copyto_list = get_additional_option(func, "copyto", [])
+                copyto_list.append(cls.__name__)
+                upgrade_signature(func, additional_options={"copyto": copyto_list})
+            else:
+                upgrade_signature(func, additional_options={"into": cls.__name__})
             return method
         
         return wrapper if method is None else wrapper(method)
     
     
-    def _unwrap_method(self, child_clsname: str, name: str, widget: FunctionGui | PushButtonPlus):
+    def _unwrap_method(self, 
+                       method_name: str,
+                       widget: FunctionGui | PushButtonPlus | Action,
+                       moveto: str,
+                       copyto: list[str]):
         """
         This private method converts class methods that are wrapped by its child widget class
         into widget in child widget. Practically same widget is shared between parent and child,
@@ -290,9 +321,9 @@ class MagicTemplate:
 
         Parameters
         ----------
-        child_clsname : str
+        moveto : str
             Name of child widget class name.
-        name : str
+        method_name : str
             Name of method.
         widget : FunctionGui
             Widget to be added.
@@ -303,31 +334,43 @@ class MagicTemplate:
             If ``child_clsname`` was not found in child widget list. This error will NEVER be raised
             in the user's side.
         """
+        if moveto is not None:
+            matcher = copyto + [moveto]
+        else:
+            matcher = copyto
+        
+        _found = 0
+        _n_match = len(matcher)
+        
         for child_instance in self._iter_child_magicclasses():
-            if child_instance.__class__.__name__ == child_clsname:
+            _name = child_instance.__class__.__name__
+            if _name in matcher:
                 # get the position of predefined child widget
                 try:
-                    index = _get_index(child_instance, name)
+                    index = _get_index(child_instance, method_name)
                     new = False
                 except ValueError:
                     index = -1
                     new = True
                 
-                self.append(widget)
+                self._fast_insert(-1, widget)
+                copy = _name in copyto
                 
                 if isinstance(widget, FunctionGui):
+                    if copy:
+                        widget = widget.copy()
                     if new:
-                        child_instance.append(widget)
+                        child_instance._fast_insert(-1, widget)
                     else:
                         del child_instance[index]
-                        child_instance.insert(index, widget)
+                        child_instance._fast_insert(index, widget)
                     
                 else:
-                    widget.visible = False
+                    widget.visible = copy
                     if new:
                         child_widget = child_instance._create_widget_from_method(lambda x: None)
                         child_widget.text = widget.text
-                        child_instance.append(child_widget)
+                        child_instance._fast_insert(-1, child_widget)
                     else:
                         child_widget: PushButtonPlus | AbstractAction = child_instance[index]
                     
@@ -338,9 +381,12 @@ class MagicTemplate:
                     
                 widget._unwrapped = True
                 
-                break
+                _found += 1
+                if _found == _n_match:
+                    break
+            
         else:
-            raise RuntimeError(f"{child_clsname} not found in class {self.__class__.__name__}")
+            raise RuntimeError(f"{method_name} not found in class {self.__class__.__name__}")
     
     
     def _convert_attributes_into_widgets(self):
@@ -409,7 +455,7 @@ class MagicTemplate:
         all_params = []
         for param in func.__signature__.parameters.values():
             if isinstance(param.annotation, _AnnotatedAlias):
-                # TODO: pydantic
+                # TODO: after magicgui supports pydantic, something needs update here.
                 param = MagicParameter.from_parameter(param)
             if isinstance(param, MagicParameter):
                 bound_value = param.options.get("bind", None)
@@ -500,7 +546,9 @@ class MagicTemplate:
                         mgui.insert(0, title)
                         mgui.append(Separator(orientation="horizontal"))
                         
-                        if self._popup_mode == PopUpMode.parentlast:
+                        if self._popup_mode == PopUpMode.popup:
+                            pass
+                        elif self._popup_mode == PopUpMode.parentlast:
                             parent_self = self._search_parent_magicclass()
                             parent_self.append(mgui)
                         elif self._popup_mode == PopUpMode.first:
@@ -587,6 +635,7 @@ class MagicTemplate:
 
 
     def _search_parent_magicclass(self) -> MagicTemplate:
+        """Find the ancestor."""
         current_self = self
         while getattr(current_self, "__magicclass_parent__", None) is not None:
             current_self = current_self.__magicclass_parent__
@@ -594,6 +643,7 @@ class MagicTemplate:
 
 
     def _iter_child_magicclasses(self) -> Iterable[MagicTemplate]:
+        """Iterate over all the child magic classes"""
         for child in self.__magicclass_children__:
             yield child
             yield from child.__magicclass_children__
@@ -816,8 +866,14 @@ def _build_mgui(widget_, func):
     try:
         call_button = get_additional_option(func, "call_button", None)
         layout = get_additional_option(func, "layout", "vertical")
+        labels = get_additional_option(func, "labels", True)
         auto_call = get_additional_option(func, "auto_call", False)
-        mgui = FunctionGuiPlus(func, call_button, layout=layout, auto_call=auto_call)
+        mgui = FunctionGuiPlus(func, 
+                               call_button,
+                               layout=layout,
+                               labels=labels,
+                               auto_call=auto_call
+                               )
     except Exception as e:
         msg = (
             "Exception was raised during building magicgui from method "
