@@ -10,13 +10,14 @@ from magicgui.widgets._concrete import _LabeledWidget
 from macrokit import Symbol
 
 from .mgui_ext import PushButtonPlus
-from .toolbar import ToolBarGui
+from .toolbar import ToolBarGui, QtTabToolBar
 from .menu_gui import MenuGui, ContextMenuGui
 from ._base import BaseGui, PopUpMode, ErrorMode, value_widget_callback, nested_function_gui_callback
 from .utils import define_callback, MagicClassConstructionError, define_context_menu
 from ..widgets import (
     ButtonContainer,
     GroupBoxContainer,
+    FrameContainer,
     ListContainer,
     SubWindowsContainer,
     ScrollableContainer,
@@ -35,6 +36,9 @@ from ..fields import MagicField
 from ..signature import get_additional_option
 from .._app import run_app
 
+# For Containers that belong to these classes, menubar must be set to _qwidget.layout().
+_USE_OUTER_LAYOUT = (ScrollableContainer, DraggableContainer, SplitterContainer, TabbedContainer)
+
 
 class ClassGuiBase(BaseGui):
     # This class is always inherited by @magicclass decorator.
@@ -67,11 +71,11 @@ class ClassGuiBase(BaseGui):
                 widget.changed.connect(f)
         
         elif fld.callbacks:
-            msg = (
+            warnings.warn(
                 f"{type(widget).__name__} does not have value-change callback. "
-                f"Connecting callback functions does no effect."
+                f"Connecting callback functions does no effect.",
+                UserWarning
                 )
-            warnings.warn(msg, UserWarning)
                 
         return widget
     
@@ -99,13 +103,11 @@ class ClassGuiBase(BaseGui):
             
             try:
                 if isinstance(attr, type):
+                    if not issubclass(attr, BaseGui):
+                        continue
                     # Nested magic-class
                     widget = attr()
-                    setattr(self, name, widget)
-                
-                elif isinstance(attr, BaseGui):
-                    widget = attr
-                    setattr(self, name, widget)
+                    object.__setattr__(self, name, widget)
                 
                 elif isinstance(attr, MagicField):
                     # If MagicField is given by field() function.
@@ -127,7 +129,6 @@ class ClassGuiBase(BaseGui):
                 
                 if isinstance(widget, MenuGui):
                     # Add menubar to container
-                    # widget.__magicclass_parent__ = self
                     if self._menubar is None:
                         # if widget has no menubar, a new one should be created.
                         self._menubar = QMenuBar(parent=self.native)
@@ -138,14 +139,14 @@ class ClassGuiBase(BaseGui):
                             if hasattr(self._widget, "_scroll_area"):
                                 _layout: QBoxLayout = self._widget._qwidget.layout()
                             else:
-                                _layout = self.native.layout()
+                                _layout = self._widget._layout
                             if _layout.menuBar() is None:
                                 _layout.setMenuBar(self._menubar)
                             else:
                                 raise RuntimeError(
                                     "Cannot add menubar after adding a toolbar in a non-main window. "
-                                    "Use maigcclass(widget_type='mainwindow') instead, or reorder "
-                                    "class definition."
+                                    "Use maigcclass(widget_type='mainwindow') instead, or define the"
+                                    "menu class before toolbar class."
                                     )
                     
                     widget.native.setParent(self._menubar, widget.native.windowFlags())
@@ -154,7 +155,6 @@ class ClassGuiBase(BaseGui):
                 
                 elif isinstance(widget, ContextMenuGui):
                     # Add context menu to container
-                    # widget.__magicclass_parent__ = self
                     self.native.setContextMenuPolicy(Qt.CustomContextMenu)
                     self.native.customContextMenuRequested.connect(
                         define_context_menu(widget, self.native)
@@ -162,23 +162,26 @@ class ClassGuiBase(BaseGui):
                     _hist.append((name, type(attr), "ContextMenuGui"))
                 
                 elif isinstance(widget, ToolBarGui):
-                    # widget.__magicclass_parent__ = self
-                    if issubclass(self.__class__, MainWindow):
-                        self.native: QMainWindow
-                        self.native.addToolBar(widget.native)
-                    else:
-                        # self is not a main window object
-                        if hasattr(self._widget, "_scroll_area"):
-                            _layout: QBoxLayout = self._widget._qwidget.layout()
+                    if self._toolbar is None:
+                        self._toolbar = QtTabToolBar(widget.name, self.native)
+                        if issubclass(self.__class__, MainWindow):
+                            self.native: QMainWindow
+                            self.native.addToolBar(self._toolbar)
                         else:
-                            _layout = self.native.layout()
-                        if _layout.menuBar() is None:
-                            _layout.setMenuBar(widget.native)
-                        else:
-                            _layout.insertWidget(0, widget.native, alignment=Qt.AlignTop)
-                            widget.native.setContentsMargins(0, 0, 0, 0)
-                            n_insert += 1
-                            
+                            # self is not a main window object
+                            if isinstance(self, _USE_OUTER_LAYOUT):
+                                _layout: QBoxLayout = self._widget._qwidget.layout()
+                            else:
+                                _layout = self._widget._layout
+                            if _layout.menuBar() is None:
+                                _layout.setMenuBar(self._toolbar)
+                            else:
+                                _layout.insertWidget(0, self._toolbar, alignment=Qt.AlignTop)
+                                self._toolbar.setContentsMargins(0, 0, 0, 0)
+                                n_insert += 1
+                    
+                    widget.native.setParent(self._toolbar, widget.native.windowFlags())
+                    self._toolbar.addToolBar(widget.native, widget.name.replace("_", " "))
                     _hist.append((name, type(attr), "ToolBarGui"))
                 
                 elif isinstance(widget, (Widget, Callable)):
@@ -189,7 +192,7 @@ class ClassGuiBase(BaseGui):
                         # useful. Those callable objects should be passed from widget construction.
                         try:
                             widget = self._create_widget_from_method(widget)
-                        except Exception:
+                        except AttributeError:
                             msg = f"Could not convert {widget!r} into a widget."
                             warnings.warn(msg, UserWarning)
                             continue
@@ -212,9 +215,10 @@ class ClassGuiBase(BaseGui):
                     if name.startswith("_"):
                         continue
                 
-                    clsname = get_additional_option(attr, "into")
-                    if clsname is not None:
-                        self._unwrap_method(clsname, name, widget)
+                    moveto = get_additional_option(attr, "into")
+                    copyto = get_additional_option(attr, "copyto", [])
+                    if moveto is not None or copyto:
+                        self._unwrap_method(name, widget, moveto, copyto)
                     else:
                         self._fast_insert(n_insert, widget)
                         n_insert += 1
@@ -261,10 +265,12 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
                      popup_mode: str | PopUpMode = None, 
                      error_mode: str | ErrorMode = None,
                      labels: bool = True, 
-                     name: str = None, 
+                     name: str = None,
+                     visible: bool = None,
                      ):
 
-            container.__init__(self, layout=layout, labels=labels, name=name)
+            container.__init__(self, layout=layout, labels=labels, name=name, 
+                               visible=visible)
             BaseGui.__init__(self, close_on_run=close_on_run, popup_mode=popup_mode, 
                              error_mode=error_mode)
             
@@ -272,6 +278,7 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
                 self.parent = parent
                 
             self._menubar = None
+            self._toolbar = None
             
             self.native.setObjectName(self.name)
             self.native.setWindowTitle(self.name)
@@ -379,7 +386,7 @@ def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiB
                     pass
                 
             container.close(self)
-                
+
             return None
         
         cls.__init__ = __init__
@@ -434,5 +441,8 @@ class SubWindowsClassGui: pass
 @make_gui(GroupBoxContainer, no_margin=False)
 class GroupBoxClassGui: pass
 
+@make_gui(FrameContainer, no_margin=False)
+class FrameClassGui: pass
+
 @make_gui(MainWindow)
 class MainWindowClassGui: pass
