@@ -66,9 +66,11 @@ class MenuGuiBase(ContainerLikeGui):
         for name, attr in filter(lambda x: x[0] not in base_members, iter_members(cls)):
             try:
                 if isinstance(attr, type):
+                    if not issubclass(attr, BaseGui):
+                        continue
                     # Nested magic-menu
                     widget = attr()
-                    setattr(self, name, widget)
+                    object.__setattr__(self, name, widget)
                 
                 elif isinstance(attr, MagicField):
                     widget = self._create_widget_from_field(name, attr)
@@ -111,9 +113,10 @@ class MenuGuiBase(ContainerLikeGui):
                         # with a type object (not instance).
                         widget.__magicclass_parent__ = self
                         
-                    clsname = get_additional_option(attr, "into")
-                    if clsname is not None:
-                        self._unwrap_method(clsname, name, widget)
+                    moveto = get_additional_option(attr, "into")
+                    copyto = get_additional_option(attr, "copyto", [])
+                    if moveto is not None or copyto:
+                        self._unwrap_method(name, widget, moveto, copyto)
                     else:           
                         self._fast_insert(len(self), widget)
                     
