@@ -6,7 +6,7 @@ from magicgui.widgets import Image, Table, Label, FunctionGui
 from magicgui.widgets._bases import ButtonWidget
 from magicgui.widgets._bases.widget import Widget
 from macrokit import Symbol
-from qtpy.QtWidgets import QToolBar, QMenu, QWidgetAction
+from qtpy.QtWidgets import QToolBar, QMenu, QWidgetAction, QTabWidget
 from qtpy.QtCore import Qt
 
 from .mgui_ext import AbstractAction, _LabeledWidgetAction, WidgetAction, ToolButtonPlus
@@ -36,6 +36,27 @@ def _check_popupmode(popup_mode: PopUpMode):
     return popup_mode
 
 
+class QtTabToolBar(QToolBar):
+    """ToolBar widget with tabs."""
+    def __init__(self, title: str, parent=None):
+        super().__init__(title, parent)
+        self._tab = QTabWidget(self)
+        self._tab.setContentsMargins(0, 0, 0, 0)
+        self._tab.setTabBarAutoHide(True)
+        self._tab.setStyleSheet(
+            "QTabWidget {"
+            "    margin: 0px, 0px, 0px, 0px;"
+            "    padding: 0px;}"
+            )
+        self.addWidget(self._tab)
+    
+    def addToolBar(self, toolbar: QToolBar, name: str) -> None:
+        """Add a toolbar as a tab."""
+        self._tab.addTab(toolbar, name)
+        toolbar.setContentsMargins(0, 0, 0, 0)
+        return None
+
+
 class ToolBarGui(ContainerLikeGui):
     """Magic class that will be converted into a toolbar"""
     
@@ -89,9 +110,11 @@ class ToolBarGui(ContainerLikeGui):
         for name, attr in filter(lambda x: x[0] not in base_members, iter_members(cls)):
             try:
                 if isinstance(attr, type):
+                    if not issubclass(attr, BaseGui):
+                        continue
                     # Nested magic-menu
                     widget = attr()
-                    setattr(self, name, widget)
+                    object.__setattr__(self, name, widget)
                 
                 elif isinstance(attr, MagicField):
                     widget = self._create_widget_from_field(name, attr)
@@ -156,9 +179,10 @@ class ToolBarGui(ContainerLikeGui):
                         widget.__magicclass_parent__ = self
                     
                         
-                    clsname = get_additional_option(attr, "into")
-                    if clsname is not None:
-                        self._unwrap_method(clsname, name, widget)
+                    moveto = get_additional_option(attr, "into")
+                    copyto = get_additional_option(attr, "copyto", [])
+                    if moveto is not None or copyto:
+                        self._unwrap_method(name, widget, moveto, copyto)
                     else:           
                         self.insert(len(self), widget)
                     
