@@ -8,6 +8,7 @@ from magicgui.widgets import * # to avoid importing both magicgui.widgets and ma
 from .containers import (
     ButtonContainer,
     GroupBoxContainer,
+    FrameContainer,
     ListContainer,
     SubWindowsContainer,
     ScrollableContainer,
@@ -23,7 +24,6 @@ from .pywidgets import ListWidget, DictWidget
 from .misc import Figure, ConsoleTextEdit, CheckButton
 from .separator import Separator
 from .sequence import ListEdit, TupleEdit
-from .threading import ProgressWidget, progress
 from .utils import FreeWidget
 
 
@@ -35,11 +35,10 @@ __all__ = ["ListWidget",
            "ListEdit",
            "DictWidget",
            "TupleEdit",
-           "ProgressWidget",
-           "progress",
            "FreeWidget",
            "ButtonContainer",
            "GroupBoxContainer",
+           "FrameContainer",
            "ListContainer",
            "SubWindowsContainer",
            "ScrollableContainer",
