@@ -14,7 +14,7 @@ from magicgui.backends._qtpy.widgets import (
 
 # Container variations that is useful in making GUI designs better.
 
-C = TypeVar("C")
+C = TypeVar("C", bound=ContainerWidget)
 
 def wrap_container(cls: type[C] = None, base: type = None) -> Callable | type[C]:
     """
@@ -34,21 +34,27 @@ def wrap_container(cls: type[C] = None, base: type = None) -> Callable | type[C]
     return wrapper(cls) if cls else wrapper
 
 class _Splitter(ContainerBase):
-    _qwidget: QtW.QSplitter
+    _qwidget: QtW.QWidget
     def __init__(self, layout="vertical"):
-        QBaseWidget.__init__(self, QtW.QSplitter)
+        QBaseWidget.__init__(self, QtW.QWidget)
         # SetLayout is not supported for QSplitter.
         # Layout is just a dummy.
+        self._splitter = QtW.QSplitter(self._qwidget)
         if layout == "horizontal":
-            self._qwidget.setOrientation(Qt.Horizontal)
+            self._splitter.setOrientation(Qt.Horizontal)
             self._layout = QtW.QHBoxLayout()
         else:
-            self._qwidget.setOrientation(Qt.Vertical)
+            self._splitter.setOrientation(Qt.Vertical)
             self._layout = QtW.QVBoxLayout()
+        
+        self._scroll_area = None
+        self._qwidget.setLayout(QtW.QVBoxLayout())
+        self._qwidget.layout().addWidget(self._splitter)
+        self._qwidget.layout().setContentsMargins(0, 0, 0, 0)
                 
     
     def _mgui_insert_widget(self, position: int, widget: Widget):
-        self._qwidget.insertWidget(position, widget.native)
+        self._splitter.insertWidget(position, widget.native)
 
     def _mgui_remove_widget(self, widget: Widget):
         widget.native.setParent(None)
@@ -84,24 +90,27 @@ class _ToolBox(ContainerBase):
             raise ValueError(f"Widget {widget.name} not found.")
 
 class _Tab(ContainerBase):
-    _qwidget: QtW.QTabWidget
     def __init__(self, layout="vertical"):
-        QBaseWidget.__init__(self, QtW.QTabWidget)
+        QBaseWidget.__init__(self, QtW.QWidget)
         
         if layout == "horizontal":
             self._layout: QtW.QLayout = QtW.QHBoxLayout()
         else:
             self._layout = QtW.QVBoxLayout()
-        
-        self._qwidget.setLayout(self._layout)
+        self._scroll_area = None
+        self._tab_widget = QtW.QTabWidget(self._qwidget)
+        self._tab_widget.setLayout(self._layout)
+        self._qwidget.setLayout(QtW.QVBoxLayout())
+        self._qwidget.layout().addWidget(self._tab_widget)
+        self._qwidget.layout().setContentsMargins(0, 0, 0, 0)
     
     def _mgui_insert_widget(self, position: int, widget: Widget):
-        self._qwidget.insertTab(position, widget.native, widget.name)
+        self._tab_widget.insertTab(position, widget.native, widget.name)
     
     def _mgui_remove_widget(self, widget: Widget):
-        for i in range(self._qwidget.count()):
-            if self._qwidget.widget(i) is widget.native:
-                self._qwidget.removeTab(i)
+        for i in range(self._tab_widget.count()):
+            if self._tab_widget.widget(i) is widget.native:
+                self._tab_widget.removeTab(i)
                 widget.native.setParent(None)
                 break
         else:
@@ -155,7 +164,7 @@ class _WheelDisabledScrollArea(QtW.QScrollArea):
         if event.type() == QEvent.Wheel:
             return True
         return super().eventFilter(source, event)
-
+    
 class _DraggableContainer(ContainerBase):
     def __init__(self, layout="vertical"):
         QBaseWidget.__init__(self, QtW.QWidget)
@@ -214,6 +223,7 @@ class _QCollapsible(QtW.QWidget):
             return super().sizeHint()
         else:
             return QSize(0, 0)
+
             
 class _Collapsibles(ContainerBase):
     _setting: dict[str, Any]
@@ -240,7 +250,12 @@ class _Collapsibles(ContainerBase):
         self._expand_btn.setText(btn_text)
         self._expand_btn.setCheckable(True)
         self._expand_btn.setChecked(False)
-        self._expand_btn.setStyleSheet(f"QToolButton {{ border: none; text-align: {self._setting['text-align']};}}")
+        self._expand_btn.setStyleSheet(f"""
+            QToolButton {{ 
+                border: none; 
+                text-align: {self._setting['text-align']};
+                }}
+            """)
         self._expand_btn.clicked.connect(self._mgui_change_expand)
         self._mgui_change_expand()
         
@@ -250,19 +265,22 @@ class _Collapsibles(ContainerBase):
     
     @property
     def collapsed(self) -> bool:
-        return self._expand_btn.isChecked()
+        return not self._expand_btn.isChecked()
+    
+    def _collapse(self):
+        self._inner_widget.setVisible(False)
+        self._expand_btn.setArrowType(self._setting["collapsed-arrow"])
+
+    def _expand(self):
+        self._inner_widget.setVisible(True)
+        self._expand_btn.setArrowType(self._setting["expanded-arrow"])
     
     def _mgui_change_expand(self):
-        if not self.collapsed:
-            # collapse
-            self._inner_widget.setVisible(False)
-            self._expand_btn.setArrowType(self._setting["collapsed-arrow"])
-
+        if self.collapsed:
+            self._collapse()
         else:
-            # expand
-            self._inner_widget.setVisible(True)
-            self._expand_btn.setArrowType(self._setting["expanded-arrow"])
-    
+            self._expand()
+
     def _get_setting(self):
         raise NotImplementedError()
 
@@ -333,6 +351,7 @@ class _SubWindowsContainer(ContainerBase):
         self._qwidget.setLayout(self._layout)
     
     def _mgui_insert_widget(self, position: int, widget: Widget):
+        # position does not have any effect
         self._mdiarea.addSubWindow(widget.native, self._NoCloseButtonFlag)
     
     def _mgui_remove_widget(self, widget: Widget):
@@ -355,13 +374,18 @@ class _GroupBoxContainer(ContainerBase):
         self._groupbox.setLayout(QtW.QHBoxLayout())
         self._groupbox.layout().addWidget(self._inner_widget)
         
-        self._qwidget.setLayout(QtW.QHBoxLayout())
+        self._qwidget.setLayout(QtW.QVBoxLayout())
         self._qwidget.layout().addWidget(self._groupbox)
         self._qwidget.layout().setContentsMargins(0, 0, 0, 0)
     
     def _set_title(self, title: str):
         self._groupbox.setTitle(title)
 
+class _FrameContainer(_GroupBoxContainer):
+    def __init__(self, layout="vertical"):
+        super().__init__(layout=layout)
+        self._groupbox.setTitle("")
+
 @wrap_container(base=_Splitter)
 class SplitterContainer(ContainerWidget):
     """A Container equipped with splitter"""
@@ -384,11 +408,11 @@ class TabbedContainer(ContainerWidget):
     
     @property
     def current_index(self):
-        return self.native.currentIndex()
+        return self._widget._tab_widget.currentIndex()
     
     @current_index.setter
     def current_index(self, index: int):
-        self.native.setCurrentIndex(index)
+        self._widget._tab_widget.setCurrentIndex(index)
 
 @wrap_container(base=_Stack)
 class StackedContainer(ContainerWidget):
@@ -441,7 +465,10 @@ class CollapsibleContainer(ContainerWidget):
     
     @collapsed.setter
     def collapsed(self, value: bool):
-        self._widget._expand_btn.setChecked(value)
+        if value:
+            self._widget._collapse()
+        else:
+            self._widget._expand()
 
 
 @wrap_container(base=_HCollapsibleContainer)
@@ -462,7 +489,10 @@ class HCollapsibleContainer(ContainerWidget):
     
     @collapsed.setter
     def collapsed(self, value: bool):
-        self._widget._expand_btn.setChecked(value)
+        if value:
+            self._widget._collapse()
+        else:
+            self._widget._expand()
 
 
 @wrap_container(base=_ListContainer)
@@ -510,4 +540,8 @@ class GroupBoxContainer(ContainerWidget):
     @name.setter
     def name(self, value: str):
         self._name = value
-        self._widget._set_title(value.replace("_", " "))
\ No newline at end of file
+        self._widget._set_title(value.replace("_", " "))
+
+@wrap_container(base=_FrameContainer)
+class FrameContainer(ContainerWidget):
+    """A QGroupBox like container without title."""
\ No newline at end of file
