@@ -1,12 +1,15 @@
 from __future__ import annotations
 from functools import wraps
+import inspect
 from typing import Callable, Iterable, Iterator, Union, TYPE_CHECKING, TypeVar, overload
-from magicgui.widgets._bases import ButtonWidget
+from magicgui.widgets import Label
+from macrokit import Expr, Symbol, Head
 from .signature import upgrade_signature
 
 if TYPE_CHECKING:
     from .gui import BaseGui
     from .gui.mgui_ext import Action
+    from magicgui.widgets._bases import ButtonWidget
 
 Color = Union[str, Iterable[float]]
 nStrings = Union[str, Iterable[str]]
@@ -16,6 +19,7 @@ T = TypeVar("T")
 
 
 def set_options(layout: str = "vertical", 
+                labels: bool = True,
                 call_button: bool | str | None = None,
                 auto_call: bool = False,
                 **options) -> Callable[[Callable[[Args], Returns]], Callable[[Args], Returns]]:
@@ -27,9 +31,10 @@ def set_options(layout: str = "vertical",
     
     Parameters
     ----------
-    layout : str, optional
-        The type of layout to use. Must be one of {'horizontal', 'vertical'}.
-        by default "vertical".
+    layout : str, default is "vertical"
+        The type of layout to use in FunctionGui. Must be one of {'horizontal', 'vertical'}.
+    labels : bool, default is True
+        Whether labels are shown in the FunctionGui.
     call_button : bool or str, optional
         If ``True``, create an additional button that calls the original
         function when clicked.  If a ``str``, set the button text. If None (the
@@ -46,6 +51,7 @@ def set_options(layout: str = "vertical",
                           gui_options=options,
                           additional_options={"call_button": call_button,
                                               "layout": layout,
+                                              "labels": labels,
                                               "auto_call": auto_call}
                           )
         return func
@@ -186,6 +192,44 @@ def bind_key(*key) -> Callable[[Callable[[Args], Returns]], Callable[[Args], Ret
     return wrapper
 
 
+def confirm(text: str, call_button_text: str = "OK"):
+    """
+    Confirm if it is OK to run function in GUI. Useful when the function will irreversibly
+    delete or update something in GUI.
+
+    Parameters
+    ----------
+    text : str
+        Confirmation text, such as "Are you sure to run this function?".
+    call_button_text : str, default is "OK"
+        Text shown on the call button.
+    """
+    if not isinstance(text, str):
+        raise TypeError(
+            f"The first argument of 'confirm' must be a str bug got {type(text)}"
+            )
+    def _decorator(method: Callable[[Args], Returns]) -> Callable[[Args], Returns]:
+        sig = inspect.signature(method)
+        if len(sig.parameters) != 1:
+            raise ValueError(
+                "Currently 'confirm' only supports methods that take no argument."
+                )
+        @do_not_record
+        @set_options(call_button=call_button_text,
+                     labels=False,
+                     label={"widget_type": Label, "value": text})
+        def _method(self: BaseGui, label):
+            out = method(self)
+            expr = Expr.parse_method(self, method, (), {})
+            self.macro.append(expr)
+            return out
+        _method.__name__ = method.__name__
+        _method.__qualname__ = method.__qualname__
+        _method.__module__ = method.__module__
+        return _method
+    return _decorator
+
+
 def _assert_iterable(obj):
     if obj is None:
         obj = []
