@@ -83,56 +83,6 @@ def test_toolbar():
     assert str(ui.macro[-1]) == "ui.f1()"
     ui.Toolbar.Menu["inner1"].changed()
     assert str(ui.macro[-1]) == "ui.Toolbar.Menu.inner1()"
-    
-
-def test_wraps():
-    @magicclass
-    class A:
-        @magicclass
-        class B:
-            def f1(self): ...
-            def f2(self): ...
-            def f3(self): ...
-        @B.wraps
-        def f2(self):
-            self.new_attr = 0
-    
-    ui = A()
-    
-    # assert the widget order is conserved
-    assert ui.B[0].name == "f1"
-    assert ui.B[1].name == "f2"
-    assert ui.B[2].name == "f3"
-    
-    mock = MagicMock()
-    ui.changed.connect(mock)
-    ui.B["f2"].changed()
-    mock.assert_called()
-    assert hasattr(ui, "new_attr")
-    
-def test_double_wrap():
-    @magicclass
-    class A:
-        @magicclass
-        class B:
-            @magicmenu
-            class C:
-                def f1(self): ...
-                def f2(self): ...
-                def f3(self): ...
-        @B.C.wraps
-        def f2(self):
-            self.new_attr = 0
-    
-    ui = A()
-    
-    # assert the widget order is conserved
-    assert ui.B.C[0].name == "f1"
-    assert ui.B.C[1].name == "f2"
-    assert ui.B.C[2].name == "f3"
-    
-    ui.B.C["f2"].changed()
-    assert hasattr(ui, "new_attr")
 
 
 def test_separator():
