@@ -1,8 +1,20 @@
-from magicclass import magicclass, WidgetType, build_help
+from magicclass import magicclass, magicmenu, magictoolbar, WidgetType, build_help
 
 def _make_class(t: WidgetType):
     @magicclass(widget_type=t)
     class A:
+        @magicmenu
+        class Menu:
+            pass
+        
+        @magictoolbar
+        class Tool1:
+            pass
+        
+        @magictoolbar
+        class Tool2:
+            pass
+        
         @magicclass
         class B:
             def b1(self, a: int): ...
@@ -24,7 +36,6 @@ def test_all_works():
         ui[1].changed()
         ui.B[0].changed()
         ui.B[1].changed()
+        if hasattr(ui, "current_index"):
+            ui.current_index = 1
         ui.close()
-        help = build_help(ui)
-        help.show()
-        help.close()
\ No newline at end of file
